@@ -15,9 +15,6 @@
 #' If input is a logical value, it is equivalent to \code{weight = "probability"} if \code{weight = TRUE} while 
 #' equivalent to \code{weight = "constant"} if \code{weight = FALSE}.
 #' Default: \code{weight = FALSE}.
-## @param type If \code{type = 'bcor'}, Ball Correlation will be used instead of Ball Covariance.(default \code{type = 'bcov'})
-## @param method if \code{method = 'permute'}, a permutation procedure will be carried out;
-## if \code{method = 'approx'}, the p-values based on approximate Ball Covariance distribution are given.(Test arguments)
 #' 
 #' @return If \code{num.permutations > 0}, \code{bcov.test} returns a \code{htest} class object containing the following components:
 #' \item{\code{statistic}}{Ball Covariance statistic.}            
@@ -32,7 +29,7 @@
 #' If \code{num.permutations = 0}, \code{bcov.test} returns a statistic value.
 #' 
 #' @details 
-#' \code{bcov.test} are non-parametric tests of independence in Banach spaces. 
+#' \code{bcov.test} is non-parametric tests of independence in Banach spaces. 
 #' It can detect the dependence between two random objects (variables) and 
 #' the mutual dependence among at least three random objects (variables).
 #' 
@@ -66,9 +63,9 @@
 #' set.seed(1)
 #' 
 #' ################# Quick Start #################
-#' error <- runif(50, min = -0.3, max = 0.3)
+#' noise <- runif(50, min = -0.3, max = 0.3)
 #' x <- runif(50, 0, 4*pi)
-#' y <- cos(x) + error
+#' y <- cos(x) + noise
 #' # plot(x, y)
 #' res <- bcov.test(x, y)
 #' res
@@ -79,8 +76,8 @@
 #' 
 #' ################# Quick Start #################
 #' x <- matrix(runif(50 * 2, -pi, pi), nrow = 50, ncol = 2)
-#' error <- runif(50, min = -0.1, max = 0.1)
-#' y <- sin(x[,1] + x[,2]) + error
+#' noise <- runif(50, min = -0.1, max = 0.1)
+#' y <- sin(x[,1] + x[,2]) + noise
 #' bcov.test(x = x, y = y, weight = "prob")
 #' 
 #' ################# Ball Covariance Test for Non-Hilbert Data #################
@@ -115,6 +112,16 @@
 #' ################# Mutual Independence Test for Meteorology data #################
 #' data("meteorology")
 #' bcov.test(meteorology)
+#' 
+#' ################  Testing via approximate limit distribution  #################
+#' \dontrun{
+#' set.seed(1)
+#' n <- 2000
+#' x <- rnorm(n)
+#' y <- rnorm(n)
+#' bcov.test(x, y, method = "limit")
+#' bcov.test(x, y)
+#' }
 bcov.test <- function(x, ...) UseMethod("bcov.test")
 
 
@@ -122,21 +129,23 @@ bcov.test <- function(x, ...) UseMethod("bcov.test")
 #' @export
 #' @method bcov.test default
 bcov.test.default <- function(x, y = NULL, num.permutations = 99, 
+                              method = c("permutation", "limit"), 
                               distance = FALSE, weight = FALSE, 
                               seed = 1, num.threads = 0, ...)
 {
-  method = 'permute'
+  method <- match.arg(method)
   data_name <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
   if (length(data_name) > 1) {
     data_name <- ""
   }
   type <- "bcov"
   # modify input information:
-  if(class(x) == "list") {
+  if(class(x)[1] == "list") {
     data_name <- gsub(x = data_name, pattern = " and NULL", replacement = "")
   }
   weight <- examine_weight_arguments(weight)
-  result <- bcov_test_internal_wrap(x = x, y = y, num.permutations = num.permutations, distance = distance, weight = weight, 
+  result <- bcov_test_internal_wrap(x = x, y = y, num.permutations = num.permutations, 
+                                    distance = distance, weight = weight, 
                                     seed = seed, method = method, type = type, 
                                     num.threads = num.threads)
   # return result of hypothesis test:
@@ -158,11 +167,17 @@ bcov.test.default <- function(x, y = NULL, num.permutations = 99,
     } 
     
     data_name <- paste0(data_name,"\nnumber of observations = ", result[["info"]][["N"]])
-    data_name <- paste0(data_name, "\nreplicates = ", num.permutations, 
-                        ", weight: ", weight_name)
-    test_method <- "Ball Covariance test of %sindependence"
-    test_type <- ifelse(class(x) == "list" && length(x) > 2, "mutual ", "")
-    test_method <- sprintf(test_method, test_type)
+    if (method == "limit") {
+      null_method <- "Limit Distribution"
+      data_name <- paste0(data_name, "\nreplicates = ", 0)
+    } else {
+      null_method <- "Permutation"
+      data_name <- paste0(data_name, "\nreplicates = ", num.permutations)
+    }
+    data_name <- paste0(data_name, ", weight: ", weight_name)
+    test_method <- "Ball Covariance test of %sindependence (%s)"
+    test_type <- ifelse(class(x)[1] == "list" && length(x) > 2, "mutual ", "")
+    test_method <- sprintf(test_method, test_type, null_method)
     # if(type == "bcor") {
     #   test_method <- gsub(pattern = "Covariance", replacement = "Correlation", x = test_method)
     #   data_name <- gsub(pattern = "Covariance", replacement = "Correlation", x = data_name)
@@ -236,17 +251,17 @@ bcov.test.formula <- function(formula, data, subset, na.action, ...) {
 #'
 #' @noRd
 bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, weight = FALSE, 
-                               seed = 4, method = 'permute', num.threads)
+                               seed = 4, method = "permute", num.threads)
 {
-  if (class(x) == "dist" && class(y) == "dist") {
+  if (class(x)[1] == "dist" && class(y)[1] == "dist") {
     distance <- TRUE
   }
   if (distance) {
-    if (class(x) == "dist" || class(y) == "dist") {
-      if (class(x) != "dist") {
+    if (class(x)[1] == "dist" || class(y)[1] == "dist") {
+      if (class(x)[1] != "dist") {
         x <- as.dist(x)
       }
-      if (class(y) != "dist") {
+      if (class(y)[1] != "dist") {
         y <- as.dist(y)
       }
       num <- attr(x, "Size")
@@ -261,7 +276,7 @@ bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, we
     x <- as.matrix(x)
     y <- as.matrix(y)
     num <- nrow(x)
-    if (ncol(x) != 1 || ncol(y) != 1) {
+    if ((ncol(x) != 1 || ncol(y) != 1) || (method == "limit")) {
       x <- as.vector(dist(x))
       y <- as.vector(dist(y))
       distance <- TRUE
@@ -277,7 +292,7 @@ bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, we
   ## examine test type:
   # type <- examine_type_arguments(type)
   ## examine num.permutations arguments:
-  if(method == "approx") {
+  if(method == "limit") {
     num.permutations <- 0
   } else {
     examine_R_arguments(num.permutations)
@@ -286,9 +301,10 @@ bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, we
   if(num.permutations == 0) {
     result <- bcov_test_wrap_c(x = x, y = y, n = num, num.permutations = 0, 
                                distance = distance, num.threads = num.threads)
-    if(method == "approx") {
-      pvalue <- calculatePvalue(result[["statistic"]] * result[["info"]][["N"]], 
-                                BITestNullDistribution)
+    if(method == "limit") {
+      eigenvalue <- bcov_limit_wrap_c(x, y, num, distance, num.threads)
+      result[["p.value"]] <- 1 - hbe(eigenvalue, num * result[["statistic"]][1])
+      return(result)
     } else {
       if (weight == WEIGHT_TYPE[1]) {
         return(result[[1]][1])
@@ -368,11 +384,11 @@ kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weig
   #################### C Version (1.2.0) #####################
   ############################################################
   var_num <- length(x)
-  if ((!distance) && all(sapply(x, class) == "dist")) {
+  if ((!distance) && all(sapply(x, function(xx) { class(xx)[1] }) == "dist")) {
     distance <- TRUE
   }
   if(distance) {
-    if (class(x[[1]]) != "dist") {
+    if (class(x[[1]])[1] != "dist") {
       if (nrow(x[[1]]) != ncol(x[[1]])) {
         stop("The elements of input list is not a distance matrix.")
       }
@@ -381,24 +397,34 @@ kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weig
     x <- lapply(x, dist)
   }
   distance <- TRUE
-  num <- ifelse(class(x[[1]]) == "dist", attr(x[[1]], "Size"), nrow(x[[1]]))
-  if (class(x[[1]]) == "dist") {
+  num <- ifelse(class(x[[1]])[1] == "dist", attr(x[[1]], "Size"), nrow(x[[1]]))
+  if (class(x[[1]])[1] == "dist") {
     x <- lapply(x, as.vector)
   } else {
-    x <- lapply(x, function(xx) { xx[lower.tri(xx)] })
+    x <- lapply(x, function(xx) { 
+      xx[lower.tri(xx)] 
+    })
   }
   if (length(unique(sapply(x, length))) != 1) {
     stop("sample size of variables are not match!")
   } else {
+    old_x <- x
     x <- unlist(x)
   }
   #
+  if(method == "limit") {
+    num.permutations <- 0
+  } else {
+    examine_R_arguments(num.permutations)
+  }
+  #
   if(num.permutations == 0) {
     result <- kbcov_test_wrap_c(x = x, K = var_num, n = num, num.permutations = 0, 
                                 distance = distance, num.threads = num.threads)
-    if(method == "approx") {
-      pvalue <- calculatePvalue(result[["statistic"]] * result[["info"]][["N"]], 
-                                BITestNullDistribution)
+    if(method == "limit") {
+      eigenvalue <- bcov_limit_wrap_c(old_x, NULL, num, distance, num.threads)
+      result[["p.value"]] <- 1 - hbe(eigenvalue, num * result[["statistic"]])
+      return(result)
     } else {
       if (weight == WEIGHT_TYPE[1]) {
         return(result[[1]][1])
@@ -488,7 +514,7 @@ kbcov_stat <- function(x, num, var_num, weight, type) {
 bcov_test_internal_wrap <- function(x = x, y = y, num.permutations, distance, seed, 
                                     weight, method, type, num.threads)
 {
-  if(class(x) == "list") {
+  if(class(x)[1] == "list") {
     if (length(x) > 2)
     {
       result <- kbcov_test_internal(x = x, num.permutations = num.permutations, distance = distance, weight = weight, 
