@@ -8,7 +8,7 @@ test_that("Ball Correlation based Interaction Pursuit is unreasonable!", {
   p <- 3000
   n <- 150
   x <- matrix(rnorm(n = n * p), nrow = n, ncol = p)
-  y <- 4*x[, 1]*x[, 2]
+  y <- 4 * x[, 1] * x[, 2]
   
   res <- bcorsis(x = x, y = y, method = "interaction")
   expect_true(all(c(1, 2) %in% head(res[["ix"]], 5)))
@@ -75,4 +75,26 @@ test_that("Ball Correlation based SIS with Chi-square weight is unreasonable!",
   res <- bcorsis(y = y, x = x, weight = "chisq")
   expect_true(all(c(1, 3) %in% head(res[["ix"]])))
   expect_equal("chisquare", res[["weight"]])
-})
\ No newline at end of file
+})
+
+
+test_that("Ball Correlation based SIS with categorical variables is unreasonable!", {
+  set.seed(1)
+  n <- 150
+  p <- 3000
+  x <- sapply(1:p, function(i) {
+    sample(0:2, size = n, replace = TRUE)
+  })
+  eps <- rnorm(n)
+  y <- 6 * x[, 1] - 7 * x[, 2] + 5 * x[, 3] + eps
+  res <- bcorsis(x = x, y = y, category = TRUE)
+  expect_true(all(c(1, 2, 3) %in% head(res[["ix"]])))
+  
+  x <- cbind(matrix(rnorm(n * 2), ncol = 2), x)
+  res <- bcorsis(x = x, y = y, category = c(-1, -2))
+  expect_true(all(c(3, 4, 5) %in% head(res[["ix"]])))
+  
+  x <- cbind(x[, 3:5], matrix(rnorm(n * p), ncol = p))
+  res <- bcorsis(x = x, y = y, category = 1:3)
+  expect_true(all(c(1, 2, 3) %in% head(res[["ix"]], n = 10)))
+})
